/*  Quicktex Texture Compression Library
    Copyright (C) 2021 Andrew Cassidy <drewcassidy@me.com>
    Partially derived from rgbcx.h written by Richard Geldreich <richgel99@gmail.com>
    and licenced under the public domain

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "OrderTable.h"

#include <array>

#include "../../Vector4.h"

namespace quicktex::s3tc  {
using Hash = uint16_t;

template <> std::atomic<bool> OrderTable<3>::generated = false;
template <> std::atomic<bool> OrderTable<4>::generated = false;

template <> std::mutex OrderTable<3>::table_mutex = std::mutex();
template <> std::mutex OrderTable<4>::table_mutex = std::mutex();

template <> std::array<OrderTable<3>::Hash, OrderTable<3>::HashCount> *OrderTable<3>::hashes = nullptr;
template <> std::array<OrderTable<4>::Hash, OrderTable<4>::HashCount> *OrderTable<4>::hashes = nullptr;

template <> std::array<Vector4, OrderTable<3>::OrderCount> *OrderTable<3>::factors = nullptr;
template <> std::array<Vector4, OrderTable<4>::OrderCount> *OrderTable<4>::factors = nullptr;

template <> const std::array<Vector4, 3> OrderTable<3>::Weights = {{{0, 0, 0, 4}, {1, 1, 1, 1}, {4, 0, 0, 0}}};
template <> const std::array<Vector4, 4> OrderTable<4>::Weights = {{{0, 0, 0, 9}, {1, 2, 2, 4}, {4, 2, 2, 1}, {9, 0, 0, 0}}};

template <> const std::array<Hash, 3> OrderTable<3>::SingleColorHashes = {12, 15, 89};
template <> const std::array<Hash, 4> OrderTable<4>::SingleColorHashes = {15, 700, 753, 515};

// region OrderTable3
template <>
const OrderTable<3>::OrderArray OrderTable<3>::Orders = {
    {{6, 0, 10}, {3, 6, 7},  {3, 0, 13}, {13, 3, 0}, {12, 4, 0}, {9, 1, 6},  {2, 13, 1}, {4, 7, 5},  {7, 5, 4},  {9, 6, 1},  {7, 4, 5},  {8, 6, 2},  {16, 0, 0},
     {10, 6, 0}, {2, 7, 7},  {0, 0, 16}, {0, 3, 13}, {1, 15, 0}, {0, 2, 14}, {1, 4, 11}, {15, 1, 0}, {1, 12, 3}, {9, 2, 5},  {14, 1, 1}, {8, 2, 6},  {3, 3, 10},
     {4, 2, 10}, {14, 0, 2}, {0, 14, 2}, {1, 7, 8},  {6, 6, 4},  {11, 5, 0}, {6, 4, 6},  {11, 3, 2}, {4, 3, 9},  {7, 1, 8},  {10, 4, 2}, {12, 1, 3}, {11, 0, 5},
     {9, 3, 4},  {1, 0, 15}, {9, 0, 7},  {2, 6, 8},  {12, 2, 2}, {6, 2, 8},  {6, 8, 2},  {15, 0, 1}, {4, 8, 4},  {0, 4, 12}, {8, 5, 3},  {5, 9, 2},  {11, 2, 3},
     {12, 3, 1}, {6, 3, 7},  {1, 1, 14}, {2, 9, 5},  {1, 8, 7},  {4, 10, 2}, {7, 7, 2},  {13, 1, 2}, {0, 15, 1}, {3, 2, 11}, {7, 0, 9},  {4, 4, 8},  {3, 8, 5},
     {0, 5, 11}, {13, 2, 1}, {1, 10, 5}, {4, 11, 1}, {3, 10, 3}, {5, 10, 1}, {10, 2, 4}, {0, 6, 10}, {14, 2, 0}, {11, 4, 1}, {3, 12, 1}, {1, 13, 2}, {1, 5, 10},
     {5, 11, 0}, {12, 0, 4}, {8, 1, 7},  {6, 10, 0}, {3, 13, 0}, {7, 2, 7},  {0, 7, 9},  {5, 8, 3},  {0, 12, 4}, {11, 1, 4}, {13, 0, 3}, {0, 16, 0}, {5, 7, 4},
     {10, 3, 3}, {10, 0, 6}, {0, 13, 3}, {4, 6, 6},  {2, 8, 6},  {2, 5, 9},  {7, 8, 1},  {2, 1, 13}, {2, 0, 14}, {7, 3, 6},  {5, 1, 10}, {3, 11, 2}, {5, 4, 7},
     {8, 3, 5},  {10, 5, 1}, {6, 9, 1},  {1, 3, 12}, {4, 5, 7},  {2, 2, 12}, {4, 1, 11}, {0, 8, 8},  {4, 12, 0}, {6, 5, 5},  {8, 7, 1},  {5, 5, 6},  {3, 7, 6},
     {7, 9, 0},  {4, 9, 3},  {0, 10, 6}, {8, 0, 8},  {5, 3, 8},  {10, 1, 5}, {6, 1, 9},  {7, 6, 3},  {9, 5, 2},  {0, 1, 15}, {9, 7, 0},  {2, 14, 0}, {3, 4, 9},
     {8, 4, 4},  {9, 4, 3},  {0, 9, 7},  {1, 9, 6},  {3, 9, 4},  {5, 2, 9},  {2, 3, 11}, {5, 6, 5},  {1, 14, 1}, {6, 7, 3},  {2, 4, 10}, {2, 12, 2}, {8, 8, 0},
     {2, 10, 4}, {4, 0, 12}, {0, 11, 5}, {2, 11, 3}, {1, 11, 4}, {3, 5, 8},  {5, 0, 11}, {3, 1, 12}, {1, 2, 13}, {1, 6, 9}}};
// endregion

// region OrderTable4
template <>
const OrderTable<4>::OrderArray OrderTable<4>::Orders = {
    {{0, 8, 2, 6},  {4, 3, 9, 0},  {4, 8, 1, 3},  {12, 0, 3, 1}, {11, 3, 2, 0}, {6, 4, 6, 0},  {7, 5, 0, 4},  {6, 0, 8, 2},  {1, 0, 0, 15}, {3, 0, 8, 5},
     {1, 1, 13, 1}, {13, 1, 2, 0}, {0, 14, 1, 1}, {0, 15, 1, 0}, {0, 13, 0, 3}, {16, 0, 0, 0}, {4, 3, 4, 5},  {8, 6, 0, 2},  {0, 10, 0, 6}, {10, 0, 4, 2},
     {7, 2, 1, 6},  {4, 7, 5, 0},  {1, 4, 7, 4},  {0, 14, 2, 0}, {2, 7, 2, 5},  {9, 0, 5, 2},  {9, 2, 2, 3},  {10, 0, 5, 1}, {2, 3, 7, 4},  {4, 9, 0, 3},
     {1, 5, 0, 10}, {1, 1, 6, 8},  {6, 6, 4, 0},  {11, 5, 0, 0}, {11, 2, 0, 3}, {4, 0, 10, 2}, {2, 3, 10, 1}, {1, 13, 1, 1}, {0, 14, 0, 2}, {2, 3, 3, 8},
     {12, 3, 1, 0}, {14, 0, 0, 2}, {9, 1, 3, 3},  {6, 4, 0, 6},  {1, 1, 5, 9},  {5, 9, 0, 2},  {2, 10, 1, 3}, {12, 0, 0, 4}, {4, 6, 6, 0},  {0, 6, 4, 6},
     {3, 7, 4, 2},  {0, 13, 3, 0}, {3, 10, 0, 3}, {10, 2, 1, 3}, {1, 12, 1, 2}, {2, 0, 13, 1}, {11, 0, 5, 0}, {12, 1, 3, 0}, {6, 4, 5, 1},  {10, 4, 2, 0},
     {3, 6, 1, 6},  {7, 3, 6, 0},  {10, 4, 0, 2}, {10, 0, 2, 4}, {0, 5, 9, 2},  {0, 9, 3, 4},  {6, 4, 2, 4},  {3, 4, 7, 2},  {3, 3, 5, 5},  {4, 2, 9, 1},
     {6, 2, 8, 0},  {3, 5, 3, 5},  {4, 10, 1, 1}, {10, 1, 3, 2}, {5, 7, 0, 4},  {5, 3, 7, 1},  {6, 8, 1, 1},  {8, 8, 0, 0},  {11, 1, 0, 4}, {14, 1, 0, 1},
     {9, 3, 2, 2},  {8, 2, 1, 5},  {0, 0, 2, 14}, {3, 3, 9, 1},  {10, 1, 5, 0}, {8, 3, 1, 4},  {1, 5, 8, 2},  {6, 1, 9, 0},  {3, 2, 1, 10}, {3, 11, 1, 1},
     {7, 6, 3, 0},  {9, 0, 3, 4},  {5, 2, 5, 4},  {0, 2, 3, 11}, {15, 0, 0, 1}, {0, 6, 6, 4},  {3, 4, 9, 0},  {4, 7, 0, 5},  {0, 4, 4, 8},  {0, 13, 2, 1},
     {2, 4, 1, 9},  {3, 2, 5, 6},  {10, 6, 0, 0}, {3, 5, 6, 2},  {8, 0, 4, 4},  {1, 3, 6, 6},  {7, 7, 0, 2},  {6, 1, 4, 5},  {0, 11, 1, 4}, {2, 2, 8, 4},
     {0, 1, 2, 13}, {15, 0, 1, 0}, {7, 2, 6, 1},  {8, 1, 7, 0},  {1, 8, 4, 3},  {2, 13, 1, 0}, {1, 0, 7, 8},  {14, 2, 0, 0}, {1, 8, 1, 6},  {9, 3, 3, 1},
     {0, 0, 7, 9},  {4, 4, 1, 7},  {9, 0, 6, 1},  {10, 2, 4, 0}, {1, 7, 3, 5},  {0, 3, 8, 5},  {5, 2, 4, 5},  {1, 2, 5, 8},  {0, 8, 7, 1},  {10, 3, 2, 1},
     {12, 0, 4, 0}, {2, 1, 4, 9},  {5, 2, 2, 7},  {1, 9, 3, 3},  {15, 1, 0, 0}, {6, 3, 4, 3},  {9, 5, 0, 2},  {1, 6, 9, 0},  {6, 6, 0, 4},  {13, 2, 1, 0},
     {5, 1, 8, 2},  {0, 5, 11, 0}, {7, 1, 0, 8},  {1, 2, 12, 1}, {0, 3, 3, 10}, {7, 4, 2, 3},  {5, 1, 4, 6},  {7, 0, 3, 6},  {3, 12, 0, 1}, {3, 4, 5, 4},
     {1, 10, 0, 5}, {7, 4, 3, 2},  {10, 5, 0, 1}, {13, 3, 0, 0}, {2, 5, 4, 5},  {3, 10, 1, 2}, {5, 1, 2, 8},  {14, 0, 1, 1}, {1, 5, 4, 6},  {1, 4, 5, 6},
     {2, 3, 11, 0}, {11, 0, 4, 1}, {11, 2, 2, 1}, {5, 3, 8, 0},  {1, 3, 10, 2}, {0, 1, 13, 2}, {3, 1, 4, 8},  {4, 2, 4, 6},  {1, 5, 6, 4},  {2, 1, 11, 2},
     {1, 2, 9, 4},  {4, 7, 3, 2},  {6, 2, 5, 3},  {7, 2, 2, 5},  {8, 1, 4, 3},  {3, 2, 8, 3},  {12, 1, 0, 3}, {7, 8, 1, 0},  {7, 0, 2, 7},  {5, 10, 0, 1},
     {0, 2, 14, 0}, {2, 9, 3, 2},  {7, 0, 0, 9},  {11, 1, 4, 0}, {10, 4, 1, 1}, {2, 2, 9, 3},  {5, 7, 2, 2},  {1, 3, 1, 11}, {13, 2, 0, 1}, {4, 2, 8, 2},
     {2, 3, 1, 10}, {4, 2, 5, 5},  {7, 0, 7, 2},  {10, 0, 0, 6}, {0, 8, 5, 3},  {4, 4, 0, 8},  {12, 4, 0, 0}, {0, 1, 14, 1}, {8, 0, 1, 7},  {5, 1, 5, 5},
     {11, 0, 3, 2}, {0, 4, 1, 11}, {0, 8, 8, 0},  {0, 2, 5, 9},  {7, 3, 2, 4},  {7, 8, 0, 1},  {1, 0, 3, 12}, {7, 4, 5, 0},  {1, 6, 7, 2},  {7, 6, 1, 2},
     {9, 6, 1, 0},  {12, 2, 0, 2}, {4, 1, 6, 5},  {4, 0, 1, 11}, {8, 4, 4, 0},  {13, 0, 1, 2}, {8, 6, 2, 0},  {4, 12, 0, 0}, {2, 7, 5, 2},  {2, 0, 5, 9},
     {5, 4, 5, 2},  {3, 8, 5, 0},  {7, 3, 3, 3},  {4, 4, 8, 0},  {2, 1, 3, 10}, {5, 0, 1, 10}, {6, 4, 3, 3},  {4, 9, 1, 2},  {1, 4, 0, 11}, {11, 3, 1, 1},
     {4, 0, 12, 0}, {13, 0, 0, 3}, {6, 1, 6, 3},  {9, 0, 4, 3},  {8, 0, 0, 8},  {8, 4, 0, 4},  {0, 12, 1, 3}, {0, 4, 10, 2}, {3, 4, 8, 1},  {1, 3, 8, 4},
     {9, 2, 5, 0},  {5, 7, 4, 0},  {1, 0, 11, 4}, {4, 10, 0, 2}, {1, 3, 12, 0}, {6, 9, 0, 1},  {5, 0, 9, 2},  {5, 9, 2, 0},  {13, 1, 0, 2}, {9, 3, 4, 0},
     {9, 4, 0, 3},  {3, 1, 12, 0}, {2, 4, 3, 7},  {1, 2, 13, 0}, {2, 2, 4, 8},  {6, 8, 0, 2},  {9, 2, 1, 4},  {9, 5, 1, 1},  {2, 0, 4, 10}, {5, 4, 0, 7},
     {0, 0, 6, 10}, {1, 2, 0, 13}, {4, 7, 2, 3},  {6, 5, 5, 0},  {3, 3, 1, 9},  {1, 6, 1, 8},  {12, 2, 1, 1}, {4, 4, 5, 3},  {1, 0, 6, 9},  {0, 6, 10, 0},
     {4, 8, 3, 1},  {4, 3, 2, 7},  {2, 1, 7, 6},  {1, 9, 1, 5},  {3, 1, 3, 9},  {8, 7, 1, 0},  {1, 2, 3, 10}, {14, 1, 1, 0}, {5, 4, 4, 3},  {3, 7, 0, 6},
     {7, 4, 1, 4},  {3, 7, 5, 1},  {1, 1, 0, 14}, {0, 10, 3, 3}, {0, 4, 3, 9},  {1, 7, 7, 1},  {2, 0, 10, 4}, {5, 8, 0, 3},  {6, 7, 3, 0},  {0, 8, 4, 4},
     {5, 7, 3, 1},  {7, 9, 0, 0},  {7, 6, 2, 1},  {0, 4, 5, 7},  {6, 3, 5, 2},  {1, 2, 1, 12}, {5, 2, 0, 9},  {8, 5, 0, 3},  {4, 6, 1, 5},  {1, 1, 7, 7},
     {10, 5, 1, 0}, {1, 2, 8, 5},  {1, 8, 2, 5},  {5, 1, 0, 10}, {6, 9, 1, 0},  {13, 0, 2, 1}, {8, 3, 5, 0},  {6, 3, 6, 1},  {2, 11, 3, 0}, {3, 7, 3, 3},
     {1, 5, 2, 8},  {7, 5, 2, 2},  {0, 6, 7, 3},  {13, 1, 1, 1}, {5, 3, 4, 4},  {7, 2, 7, 0},  {5, 8, 3, 0},  {3, 13, 0, 0}, {0, 7, 9, 0},  {8, 0, 3, 5},
     {1, 3, 7, 5},  {4, 0, 2, 10}, {12, 0, 1, 3}, {1, 7, 6, 2},  {3, 9, 0, 4},  {7, 2, 0, 7},  {0, 1, 7, 8},  {2, 1, 8, 5},  {0, 13, 1, 2}, {0, 8, 1, 7},
     {5, 0, 11, 0}, {5, 6, 2, 3},  {0, 3, 0, 13}, {2, 3, 4, 7},  {5, 6, 3, 2},  {4, 2, 10, 0}, {3, 3, 7, 3},  {7, 2, 5, 2},  {1, 1, 11, 3}, {12, 3, 0, 1},
     {5, 1, 1, 9},  {1, 15, 0, 0}, {9, 7, 0, 0},  {9, 1, 2, 4},  {0, 7, 3, 6},  {3, 0, 13, 0}, {3, 0, 11, 2}, {0, 6, 5, 5},  {8, 2, 2, 4},  {6, 10, 0, 0},
     {4, 8, 4, 0},  {0, 0, 3, 13}, {0, 4, 12, 0}, {7, 1, 6, 2},  {3, 5, 0, 8},  {8, 0, 6, 2},  {6, 2, 3, 5},  {2, 10, 0, 4}, {4, 11, 0, 1}, {6, 1, 5, 4},
     {5, 1, 3, 7},  {0, 11, 3, 2}, {4, 6, 0, 6},  {2, 6, 0, 8},  {3, 1, 7, 5},  {2, 14, 0, 0}, {2, 9, 2, 3},  {0, 3, 4, 9},  {11, 0, 1, 4}, {13, 0, 3, 0},
     {8, 3, 0, 5},  {0, 5, 3, 8},  {5, 11, 0, 0}, {0, 1, 4, 11}, {2, 1, 9, 4},  {3, 4, 4, 5},  {7, 1, 2, 6},  {12, 2, 2, 0}, {9, 4, 1, 2},  {6, 0, 2, 8},
     {4, 6, 2, 4},  {11, 2, 3, 0}, {3, 2, 2, 9},  {10, 3, 1, 2}, {1, 1, 2, 12}, {0, 5, 2, 9},  {0, 1, 11, 4}, {6, 2, 4, 4},  {2, 8, 2, 4},  {0, 9, 4, 3},
     {11, 0, 2, 3}, {0, 2, 11, 3}, {6, 0, 7, 3},  {0, 3, 6, 7},  {4, 5, 5, 2},  {1, 2, 6, 7},  {7, 5, 1, 3},  {9, 0, 2, 5},  {2, 6, 4, 4},  {4, 1, 9, 2},
     {4, 8, 2, 2},  {1, 12, 3, 0}, {0, 9, 6, 1},  {0, 10, 6, 0}, {3, 1, 5, 7},  {2, 13, 0, 1}, {2, 2, 1, 11}, {3, 6, 0, 7},  {5, 6, 5, 0},  {5, 5, 4, 2},
     {4, 0, 3, 9},  {3, 4, 1, 8},  {0, 11, 2, 3}, {2, 12, 1, 1}, {7, 1, 3, 5},  {7, 0, 9, 0},  {8, 0, 8, 0},  {1, 0, 2, 13}, {3, 3, 10, 0}, {2, 4, 4, 6},
     {2, 3, 8, 3},  {1, 10, 5, 0}, {7, 3, 0, 6},  {2, 9, 0, 5},  {1, 4, 6, 5},  {6, 6, 3, 1},  {5, 6, 0, 5},  {6, 3, 0, 7},  {3, 10, 2, 1}, {2, 5, 5, 4},
     {3, 8, 4, 1},  {1, 14, 0, 1}, {10, 3, 3, 0}, {3, 5, 7, 1},  {1, 1, 3, 11}, {2, 4, 0, 10}, {9, 3, 1, 3},  {5, 10, 1, 0}, {3, 0, 6, 7},  {3, 1, 9, 3},
     {11, 2, 1, 2}, {5, 3, 3, 5},  {0, 5, 1, 10}, {4, 1, 11, 0}, {10, 2, 0, 4}, {7, 6, 0, 3},  {2, 7, 0, 7},  {4, 2, 2, 8},  {6, 1, 7, 2},  {4, 9, 2, 1},
     {0, 0, 8, 8},  {3, 7, 2, 4},  {9, 6, 0, 1},  {0, 12, 4, 0}, {6, 7, 1, 2},  {0, 7, 2, 7},  {1, 0, 10, 5}, {0, 0, 14, 2}, {2, 7, 3, 4},  {5, 0, 0, 11},
     {7, 7, 1, 1},  {6, 2, 7, 1},  {4, 5, 3, 4},  {3, 5, 1, 7},  {5, 9, 1, 1},  {6, 2, 1, 7},  {3, 2, 0, 11}, {0, 11, 0, 5}, {3, 11, 2, 0}, {10, 1, 4, 1},
     {7, 0, 4, 5},  {11, 4, 0, 1}, {10, 3, 0, 3}, {0, 2, 4, 10}, {0, 15, 0, 1}, {0, 11, 5, 0}, {6, 7, 2, 1},  {1, 12, 2, 1}, {4, 1, 3, 8},  {1, 0, 13, 2},
     {1, 8, 5, 2},  {7, 0, 1, 8},  {3, 12, 1, 0}, {9, 2, 4, 1},  {1, 7, 4, 4},  {11, 4, 1, 0}, {4, 3, 8, 1},  {2, 8, 4, 2},  {1, 11, 3, 1}, {1, 1, 4, 10},
     {4, 10, 2, 0}, {8, 2, 5, 1},  {1, 0, 9, 6},  {5, 3, 2, 6},  {0, 9, 7, 0},  {10, 2, 2, 2}, {5, 8, 1, 2},  {8, 7, 0, 1},  {0, 3, 12, 1}, {1, 0, 1, 14},
     {4, 8, 0, 4},  {3, 8, 0, 5},  {4, 6, 5, 1},  {0, 9, 5, 2},  {10, 2, 3, 1}, {2, 3, 9, 2},  {1, 0, 12, 3}, {11, 3, 0, 2}, {4, 5, 2, 5},  {0, 2, 12, 2},
     {9, 1, 0, 6},  {9, 2, 0, 5},  {1, 2, 7, 6},  {4, 7, 4, 1},  {0, 12, 2, 2}, {0, 0, 0, 16}, {2, 8, 3, 3},  {3, 6, 2, 5},  {0, 6, 3, 7},  {7, 5, 4, 0},
     {3, 3, 3, 7},  {3, 3, 0, 10}, {5, 0, 6, 5},  {0, 0, 10, 6}, {8, 5, 3, 0},  {8, 1, 5, 2},  {6, 0, 9, 1},  {11, 1, 2, 2}, {2, 11, 2, 1}, {9, 5, 2, 0},
     {3, 0, 4, 9},  {2, 2, 12, 0}, {2, 6, 6, 2},  {2, 1, 13, 0}, {6, 0, 5, 5},  {2, 0, 14, 0}, {2, 11, 1, 2}, {4, 4, 7, 1},  {2, 0, 11, 3}, {3, 1, 1, 11},
     {2, 9, 4, 1},  {3, 7, 6, 0},  {14, 0, 2, 0}, {1, 10, 4, 1}, {8, 0, 7, 1},  {3, 6, 5, 2},  {0, 3, 11, 2}, {2, 5, 6, 3},  {11, 1, 3, 1}, {6, 5, 3, 2},
     {3, 8, 1, 4},  {0, 2, 7, 7},  {2, 10, 2, 2}, {1, 6, 2, 7},  {11, 0, 0, 5}, {12, 1, 1, 2}, {12, 1, 2, 1}, {0, 7, 1, 8},  {0, 3, 9, 4},  {0, 2, 1, 13},
     {7, 1, 4, 4},  {10, 1, 0, 5}, {4, 0, 8, 4},  {5, 2, 7, 2},  {0, 2, 0, 14}, {4, 3, 7, 2},  {2, 7, 1, 6},  {1, 2, 2, 11}, {6, 3, 3, 4},  {1, 14, 1, 0},
     {2, 4, 6, 4},  {5, 3, 6, 2},  {5, 3, 5, 3},  {8, 4, 1, 3},  {1, 3, 0, 12}, {3, 5, 2, 6},  {1, 8, 7, 0},  {0, 7, 4, 5},  {2, 1, 6, 7},  {4, 11, 1, 0},
     {7, 2, 4, 3},  {6, 1, 3, 6},  {4, 5, 4, 3},  {2, 11, 0, 3}, {1, 5, 7, 3},  {12, 0, 2, 2}, {5, 0, 4, 7},  {1, 13, 0, 2}, {7, 7, 2, 0},  {4, 1, 7, 4},
     {4, 5, 0, 7},  {5, 0, 5, 6},  {6, 5, 4, 1},  {2, 4, 2, 8},  {1, 10, 1, 4}, {6, 3, 1, 6},  {3, 3, 8, 2},  {0, 7, 7, 2},  {4, 4, 2, 6},  {1, 1, 8, 6},
     {1, 12, 0, 3}, {2, 1, 12, 1}, {1, 9, 2, 4},  {1, 11, 0, 4}, {2, 5, 2, 7},  {10, 0, 3, 3}, {4, 6, 3, 3},  {3, 7, 1, 5},  {1, 9, 0, 6},  {7, 1, 7, 1},
     {1, 6, 5, 4},  {9, 2, 3, 2},  {6, 2, 2, 6},  {2, 2, 2, 10}, {8, 3, 3, 2},  {0, 1, 8, 7},  {2, 0, 8, 6},  {0, 3, 1, 12}, {9, 4, 2, 1},  {9, 4, 3, 0},
     {6, 2, 6, 2},  {1, 8, 0, 7},  {5, 1, 10, 0}, {0, 5, 5, 6},  {8, 2, 4, 2},  {2, 3, 2, 9},  {6, 0, 3, 7},  {2, 2, 6, 6},  {2, 6, 2, 6},  {1, 13, 2, 0},
     {9, 3, 0, 4},  {7, 3, 5, 1},  {6, 5, 2, 3},  {5, 2, 6, 3},  {2, 0, 12, 2}, {5, 7, 1, 3},  {8, 1, 3, 4},  {3, 1, 10, 2}, {1, 0, 15, 0}, {0, 8, 0, 8},
     {5, 0, 7, 4},  {4, 4, 6, 2},  {0, 1, 0, 15}, {10, 0, 1, 5}, {7, 3, 4, 2},  {4, 9, 3, 0},  {2, 5, 7, 2},  {3, 4, 2, 7},  {8, 3, 2, 3},  {5, 1, 6, 4},
     {0, 10, 2, 4}, {6, 6, 1, 3},  {6, 0, 0, 10}, {4, 4, 3, 5},  {1, 3, 9, 3},  {7, 5, 3, 1},  {3, 0, 7, 6},  {1, 8, 6, 1},  {4, 3, 0, 9},  {3, 11, 0, 2},
     {6, 0, 6, 4},  {0, 1, 3, 12}, {0, 4, 2, 10}, {5, 5, 6, 0},  {4, 1, 4, 7},  {8, 1, 6, 1},  {5, 6, 4, 1},  {8, 4, 2, 2},  {4, 3, 1, 8},  {3, 0, 2, 11},
     {1, 11, 4, 0}, {0, 8, 3, 5},  {5, 1, 7, 3},  {7, 0, 8, 1},  {4, 3, 5, 4},  {4, 6, 4, 2},  {3, 2, 4, 7},  {1, 6, 3, 6},  {0, 7, 8, 1},  {3, 0, 1, 12},
     {9, 1, 4, 2},  {7, 4, 0, 5},  {1, 7, 0, 8},  {5, 4, 1, 6},  {9, 1, 5, 1},  {1, 1, 9, 5},  {4, 1, 1, 10}, {5, 3, 0, 8},  {2, 2, 5, 7},  {4, 0, 0, 12},
     {9, 0, 7, 0},  {3, 4, 0, 9},  {0, 2, 6, 8},  {8, 2, 0, 6},  {3, 2, 6, 5},  {4, 2, 6, 4},  {3, 6, 4, 3},  {2, 8, 6, 0},  {5, 0, 3, 8},  {0, 4, 0, 12},
     {0, 16, 0, 0}, {0, 9, 2, 5},  {4, 0, 11, 1}, {1, 6, 4, 5},  {0, 1, 6, 9},  {3, 4, 6, 3},  {3, 0, 10, 3}, {7, 0, 6, 3},  {1, 4, 9, 2},  {1, 5, 3, 7},
     {8, 5, 2, 1},  {0, 12, 0, 4}, {7, 2, 3, 4},  {0, 5, 6, 5},  {11, 1, 1, 3}, {6, 5, 0, 5},  {2, 1, 5, 8},  {1, 4, 11, 0}, {9, 1, 1, 5},  {0, 0, 13, 3},
     {5, 8, 2, 1},  {2, 12, 0, 2}, {3, 3, 6, 4},  {4, 1, 10, 1}, {4, 0, 5, 7},  {8, 1, 0, 7},  {5, 1, 9, 1},  {4, 3, 3, 6},  {0, 2, 2, 12}, {6, 3, 2, 5},
     {0, 0, 12, 4}, {1, 5, 1, 9},  {2, 6, 5, 3},  {3, 6, 3, 4},  {2, 12, 2, 0}, {1, 6, 8, 1},  {10, 1, 1, 4}, {1, 3, 4, 8},  {7, 4, 4, 1},  {1, 11, 1, 3},
     {1, 2, 10, 3}, {3, 9, 3, 1},  {8, 5, 1, 2},  {2, 10, 4, 0}, {4, 2, 0, 10}, {2, 7, 6, 1},  {8, 2, 3, 3},  {1, 5, 5, 5},  {3, 1, 0, 12}, {3, 10, 3, 0},
     {8, 0, 5, 3},  {0, 6, 8, 2},  {0, 3, 13, 0}, {0, 0, 16, 0}, {1, 9, 4, 2},  {4, 1, 8, 3},  {1, 6, 6, 3},  {0, 10, 5, 1}, {0, 1, 12, 3}, {4, 0, 6, 6},
     {3, 8, 3, 2},  {0, 5, 4, 7},  {1, 0, 14, 1}, {0, 4, 6, 6},  {3, 9, 1, 3},  {3, 5, 8, 0},  {3, 6, 6, 1},  {5, 4, 7, 0},  {3, 0, 12, 1}, {8, 6, 1, 1},
     {2, 9, 5, 0},  {6, 1, 1, 8},  {4, 1, 2, 9},  {3, 9, 4, 0},  {5, 2, 9, 0},  {0, 12, 3, 1}, {1, 4, 10, 1}, {4, 0, 7, 5},  {3, 1, 2, 10}, {5, 4, 2, 5},
     {5, 5, 5, 1},  {4, 2, 3, 7},  {1, 7, 5, 3},  {2, 8, 0, 6},  {8, 1, 2, 5},  {3, 8, 2, 3},  {6, 1, 2, 7},  {3, 9, 2, 2},  {9, 0, 0, 7},  {0, 8, 6, 2},
     {8, 4, 3, 1},  {0, 2, 8, 6},  {6, 5, 1, 4},  {2, 3, 5, 6},  {2, 10, 3, 1}, {0, 7, 0, 9},  {4, 2, 7, 3},  {2, 4, 8, 2},  {7, 1, 1, 7},  {2, 4, 7, 3},
     {2, 4, 10, 0}, {0, 1, 10, 5}, {4, 7, 1, 4},  {0, 10, 4, 2}, {9, 0, 1, 6},  {1, 9, 6, 0},  {3, 3, 4, 6},  {4, 5, 7, 0},  {5, 5, 2, 4},  {2, 8, 1, 5},
     {2, 3, 6, 5},  {0, 1, 1, 14}, {3, 2, 3, 8},  {10, 1, 2, 3}, {9, 1, 6, 0},  {3, 4, 3, 6},  {2, 2, 0, 12}, {0, 0, 9, 7},  {4, 0, 9, 3},  {7, 0, 5, 4},
     {4, 5, 6, 1},  {2, 5, 1, 8},  {2, 5, 9, 0},  {3, 5, 4, 4},  {1, 3, 11, 1}, {7, 1, 5, 3},  {3, 2, 7, 4},  {1, 4, 2, 9},  {1, 11, 2, 2}, {2, 2, 3, 9},
     {5, 0, 10, 1}, {3, 2, 11, 0}, {1, 10, 3, 2}, {8, 3, 4, 1},  {3, 6, 7, 0},  {0, 7, 5, 4},  {1, 3, 3, 9},  {2, 2, 10, 2}, {1, 9, 5, 1},  {0, 5, 0, 11},
     {3, 0, 3, 10}, {0, 4, 8, 4},  {2, 7, 7, 0},  {2, 0, 2, 12}, {1, 2, 11, 2}, {6, 3, 7, 0},  {0, 6, 2, 8},  {0, 10, 1, 5}, {0, 9, 0, 7},  {6, 4, 4, 2},
     {6, 0, 1, 9},  {1, 5, 10, 0}, {5, 4, 6, 1},  {5, 5, 3, 3},  {0, 0, 4, 12}, {0, 3, 2, 11}, {1, 4, 1, 10}, {3, 0, 9, 4},  {5, 5, 0, 6},  {1, 7, 8, 0},
     {2, 0, 3, 11}, {6, 4, 1, 5},  {10, 0, 6, 0}, {0, 6, 0, 10}, {0, 4, 11, 1}, {3, 1, 6, 6},  {2, 5, 8, 1},  {0, 2, 10, 4}, {3, 1, 11, 1}, {6, 6, 2, 2},
     {1, 1, 10, 4}, {2, 1, 2, 11}, {6, 1, 8, 1},  {0, 2, 13, 1}, {0, 7, 6, 3},  {6, 8, 2, 0},  {3, 0, 0, 13}, {4, 4, 4, 4},  {6, 2, 0, 8},  {7, 3, 1, 5},
     {0, 11, 4, 1}, {6, 7, 0, 3},  {2, 6, 3, 5},  {5, 2, 1, 8},  {7, 1, 8, 0},  {5, 5, 1, 5},  {1, 8, 3, 4},  {8, 2, 6, 0},  {6, 0, 10, 0}, {5, 6, 1, 4},
     {1, 4, 4, 7},  {2, 7, 4, 3},  {1, 4, 8, 3},  {5, 4, 3, 4},  {1, 10, 2, 3}, {2, 9, 1, 4},  {2, 2, 11, 1}, {2, 5, 0, 9},  {0, 0, 1, 15}, {0, 0, 11, 5},
     {0, 4, 7, 5},  {0, 1, 15, 0}, {2, 1, 0, 13}, {0, 3, 10, 3}, {8, 0, 2, 6},  {3, 3, 2, 8},  {3, 5, 5, 3},  {1, 7, 1, 7},  {1, 3, 2, 10}, {4, 0, 4, 8},
     {2, 0, 9, 5},  {1, 1, 1, 13}, {2, 2, 7, 5},  {2, 1, 10, 3}, {4, 2, 1, 9},  {4, 3, 6, 3},  {1, 3, 5, 7},  {2, 5, 3, 6},  {1, 0, 8, 7},  {5, 0, 2, 9},
     {2, 8, 5, 1},  {1, 6, 0, 9},  {0, 0, 5, 11}, {0, 4, 9, 3},  {2, 0, 7, 7},  {1, 7, 2, 6},  {2, 1, 1, 12}, {2, 4, 9, 1},  {0, 5, 7, 4},  {6, 0, 4, 6},
     {3, 2, 10, 1}, {0, 6, 1, 9},  {2, 6, 1, 7},  {0, 5, 8, 3},  {4, 1, 0, 11}, {1, 2, 4, 9},  {4, 1, 5, 6},  {6, 1, 0, 9},  {1, 4, 3, 8},  {4, 5, 1, 6},
     {1, 0, 5, 10}, {5, 3, 1, 7},  {0, 9, 1, 6},  {2, 0, 1, 13}, {2, 0, 6, 8},  {8, 1, 1, 6},  {1, 5, 9, 1},  {0, 6, 9, 1},  {0, 3, 5, 8},  {0, 2, 9, 5},
     {5, 2, 8, 1},  {1, 1, 14, 0}, {3, 2, 9, 2},  {5, 0, 8, 3},  {0, 5, 10, 1}, {5, 2, 3, 6},  {2, 6, 7, 1},  {2, 3, 0, 11}, {0, 1, 9, 6},  {1, 0, 4, 11},
     {3, 0, 5, 8},  {0, 0, 15, 1}, {2, 4, 5, 5},  {0, 3, 7, 6},  {2, 0, 0, 14}, {1, 1, 12, 2}, {2, 6, 8, 0},  {3, 1, 8, 4},  {0, 1, 5, 10}}};
// endregion

// region BestOrderTable3
template <>
const OrderTable<3>::BestOrderArray OrderTable<3>::BestOrders = {
    {{12, 1, 3, 5, 27, 2, 4, 38, 8, 7, 16, 18, 6, 10, 41, 79, 40, 23, 46, 9, 20, 88, 22, 37, 14, 19, 24, 126, 99, 119, 35, 11},
     {7, 64, 116, 14, 94, 30, 8, 42, 1, 108, 47, 55, 137, 10, 134, 95, 96, 115, 69, 32, 63, 29, 90, 113, 11, 148, 16, 103, 19, 9, 34, 25},
     {12, 1, 0, 5, 3, 7, 4, 27, 8, 6, 38, 40, 41, 16, 18, 46, 9, 10, 20, 23, 79, 62, 14, 22, 88, 99, 37, 126, 92, 19, 120, 11},
     {16, 88, 27, 18, 46, 48, 126, 107, 79, 19, 59, 38, 37, 65, 23, 66, 0, 2, 3, 43, 12, 151, 28, 25, 5, 87, 72, 40, 1, 20, 52, 92},
     {79, 48, 88, 16, 27, 65, 18, 38, 46, 19, 37, 4, 72, 33, 126, 41, 52, 0, 12, 92, 5, 1, 2, 107, 3, 77, 23, 91, 43, 51, 22, 74},
     {1, 8, 41, 122, 10, 22, 2, 0, 87, 24, 37, 120, 38, 7, 39, 4, 5, 3, 9, 92, 62, 59, 23, 16, 104, 11, 27, 79, 19, 26, 25, 32},
     {2, 76, 99, 28, 40, 86, 93, 21, 138, 60, 6, 0, 17, 128, 145, 119, 98, 144, 141, 82, 147, 54, 67, 75, 5, 12, 27, 132, 146, 1, 38, 14},
     {47, 7, 64, 90, 1, 118, 116, 85, 57, 14, 30, 94, 50, 45, 137, 134, 8, 42, 69, 139, 55, 68, 58, 108, 95, 29, 10, 115, 0, 32, 2, 11},
     {49, 8, 10, 30, 124, 11, 32, 113, 130, 58, 125, 9, 100, 53, 104, 115, 131, 103, 24, 7, 1, 39, 45, 36, 139, 0, 137, 22, 90, 44, 114, 105},
     {9, 38, 72, 125, 49, 41, 84, 11, 13, 5, 27, 0, 16, 92, 8, 2, 65, 105, 10, 18, 48, 29, 127, 131, 36, 14, 1, 46, 111, 79, 130, 12},
     {130, 8, 10, 100, 104, 131, 49, 32, 53, 39, 30, 36, 113, 24, 11, 22, 124, 44, 83, 58, 7, 103, 1, 4, 9, 125, 5, 0, 91, 33, 115, 74},
     {114, 11, 58, 8, 120, 49, 9, 124, 142, 111, 41, 30, 10, 0, 97, 130, 62, 84, 38, 5, 72, 125, 92, 127, 100, 27, 139, 113, 13, 132, 32, 1},
     {60, 46, 28, 27, 40, 20, 0, 17, 18, 2, 126, 16, 6, 38, 86, 23, 79, 54, 1, 93, 5, 88, 41, 14, 21, 111, 7, 48, 3, 84, 72, 62},
     {72, 92, 38, 65, 84, 48, 41, 79, 27, 16, 29, 111, 88, 5, 18, 46, 1, 0, 152, 14, 37, 19, 77, 42, 132, 7, 22, 13, 119, 56, 12, 2},
     {7, 55, 1, 95, 29, 56, 64, 116, 143, 8, 14, 30, 47, 94, 152, 90, 65, 67, 10, 133, 42, 72, 146, 84, 16, 48, 6, 0, 25, 108, 77, 21},
     {27, 23, 20, 5, 0, 79, 38, 2, 3, 1, 59, 46, 4, 41, 33, 86, 37, 87, 88, 92, 7, 126, 43, 8, 22, 152, 151, 150, 149, 148, 147, 146},
     {12, 0, 1, 2, 7, 6, 3, 5, 28, 4, 8, 14, 60, 40, 17, 19, 21, 86, 126, 93, 10, 18, 9, 29, 48, 99, 65, 25, 84, 119, 72, 41},
     {60, 40, 99, 2, 54, 12, 0, 1, 19, 28, 98, 93, 6, 138, 21, 5, 27, 17, 151, 14, 76, 46, 16, 18, 38, 29, 86, 144, 107, 7, 25, 41},
     {12, 0, 1, 2, 3, 5, 6, 7, 4, 28, 8, 60, 14, 40, 16, 17, 21, 10, 19, 9, 86, 38, 126, 41, 93, 27, 29, 48, 62, 84, 79, 99},
     {0, 1, 2, 10, 5, 8, 3, 25, 4, 29, 32, 34, 63, 7, 77, 26, 16, 48, 65, 56, 14, 22, 129, 103, 72, 24, 18, 152, 140, 53, 96, 42},
     {46, 126, 18, 54, 12, 16, 1, 0, 5, 2, 27, 98, 20, 23, 6, 3, 88, 48, 28, 7, 19, 8, 4, 60, 151, 38, 37, 21, 79, 14, 65, 40},
     {76, 6, 141, 86, 119, 2, 138, 67, 28, 145, 0, 93, 17, 1, 40, 60, 146, 99, 147, 14, 21, 144, 132, 7, 5, 29, 55, 27, 16, 75, 19, 12},
     {71, 5, 51, 39, 22, 80, 0, 43, 10, 122, 8, 62, 41, 24, 104, 87, 35, 37, 2, 91, 33, 120, 36, 38, 1, 131, 9, 100, 130, 66, 3, 4},
     {126, 18, 46, 27, 20, 16, 88, 23, 12, 79, 54, 59, 48, 0, 73, 1, 37, 151, 5, 19, 28, 38, 2, 66, 60, 3, 65, 98, 14, 26, 6, 43},
     {22, 10, 8, 5, 0, 71, 35, 80, 104, 39, 24, 51, 100, 1, 62, 32, 2, 130, 11, 41, 7, 9, 53, 43, 49, 83, 122, 120, 30, 44, 37, 38},
     {1, 34, 14, 129, 53, 63, 42, 26, 121, 148, 7, 44, 96, 10, 0, 24, 100, 32, 64, 116, 140, 22, 5, 19, 29, 103, 135, 108, 8, 61, 39, 83},
     {1, 7, 34, 63, 44, 25, 135, 14, 24, 108, 22, 0, 83, 94, 5, 129, 35, 101, 47, 121, 2, 19, 42, 53, 6, 110, 103, 8, 148, 10, 16, 123},
     {12, 28, 16, 60, 18, 1, 6, 21, 14, 0, 86, 19, 2, 48, 93, 17, 38, 29, 7, 5, 65, 126, 46, 72, 41, 79, 84, 119, 40, 56, 54, 88},
     {0, 2, 12, 27, 5, 46, 38, 40, 41, 79, 88, 99, 3, 23, 1, 62, 20, 4, 22, 37, 92, 35, 18, 8, 16, 24, 10, 60, 7, 120, 98, 54},
     {1, 7, 14, 56, 8, 0, 84, 67, 10, 2, 133, 72, 42, 111, 5, 30, 21, 4, 9, 3, 25, 94, 16, 116, 47, 11, 65, 18, 132, 90, 55, 64},
     {30, 8, 124, 139, 45, 11, 58, 90, 113, 137, 7, 115, 10, 32, 1, 49, 94, 85, 9, 47, 108, 103, 0, 97, 63, 14, 50, 114, 53, 106, 100, 25},
     {65, 38, 48, 27, 16, 79, 72, 18, 88, 19, 46, 77, 84, 92, 37, 41, 0, 29, 1, 14, 12, 111, 2, 5, 31, 36, 87, 74, 105, 40, 28, 51},
     {10, 8, 30, 113, 130, 100, 53, 32, 115, 103, 104, 7, 1, 121, 39, 49, 131, 44, 24, 36, 63, 137, 34, 45, 22, 90, 108, 83, 26, 11, 94, 139},
     {51, 52, 43, 33, 5, 74, 16, 37, 71, 91, 38, 3, 36, 87, 48, 22, 4, 0, 122, 41, 39, 18, 66, 27, 79, 24, 65, 88, 59, 23, 62, 92},
     {1, 7, 63, 53, 108, 121, 94, 44, 103, 100, 14, 10, 129, 47, 32, 26, 24, 25, 148, 42, 135, 22, 0, 61, 83, 8, 39, 104, 5, 64, 115, 34},
     {1, 8, 10, 7, 5, 0, 80, 32, 62, 2, 24, 44, 53, 83, 9, 41, 30, 22, 100, 11, 14, 25, 120, 4, 26, 6, 3, 16, 122, 34, 19, 35},
     {74, 4, 36, 48, 33, 91, 39, 79, 22, 16, 65, 5, 131, 38, 24, 71, 27, 52, 0, 105, 51, 18, 88, 104, 3, 31, 10, 37, 72, 19, 41, 130},
     {59, 43, 38, 79, 23, 27, 92, 51, 0, 16, 46, 5, 18, 88, 41, 37, 66, 3, 87, 20, 48, 2, 122, 4, 22, 12, 1, 126, 19, 65, 33, 24},
     {12, 28, 1, 27, 0, 16, 2, 46, 65, 60, 21, 3, 5, 18, 6, 19, 48, 14, 4, 7, 79, 88, 86, 29, 22, 72, 93, 40, 23, 8, 17, 41},
     {22, 91, 39, 33, 24, 71, 5, 131, 36, 10, 51, 0, 130, 8, 104, 2, 35, 125, 9, 43, 52, 49, 83, 80, 100, 41, 122, 3, 37, 38, 4, 16},
     {12, 0, 1, 2, 5, 3, 4, 8, 7, 27, 18, 38, 10, 6, 16, 46, 9, 20, 41, 23, 126, 79, 22, 14, 19, 99, 88, 54, 37, 48, 62, 35},
     {12, 27, 1, 2, 3, 0, 46, 4, 38, 16, 8, 28, 7, 79, 18, 5, 84, 6, 88, 10, 14, 21, 23, 20, 40, 22, 60, 19, 9, 29, 72, 65},
     {1, 14, 7, 55, 95, 29, 8, 94, 30, 56, 10, 108, 77, 116, 152, 64, 32, 48, 63, 42, 143, 148, 16, 25, 137, 65, 11, 0, 115, 9, 19, 72},
     {37, 79, 66, 38, 16, 52, 48, 59, 43, 27, 87, 33, 41, 4, 23, 51, 3, 5, 88, 18, 92, 46, 73, 122, 22, 71, 20, 0, 65, 19, 2, 120},
     {24, 32, 83, 22, 53, 1, 8, 10, 7, 30, 35, 5, 103, 0, 100, 101, 121, 113, 34, 123, 63, 2, 44, 25, 71, 115, 80, 14, 26, 108, 51, 39},
     {97, 45, 111, 58, 85, 139, 0, 90, 47, 7, 120, 106, 142, 30, 50, 132, 41, 62, 84, 1, 119, 114, 14, 56, 117, 8, 38, 29, 2, 64, 116, 5},
     {12, 28, 16, 18, 1, 60, 6, 14, 2, 21, 0, 86, 126, 19, 48, 93, 7, 27, 17, 29, 5, 65, 54, 38, 72, 79, 84, 88, 119, 145, 8, 111},
     {118, 47, 64, 116, 57, 85, 7, 14, 50, 1, 42, 0, 45, 68, 86, 69, 2, 111, 134, 28, 90, 55, 16, 29, 56, 48, 84, 144, 60, 30, 112, 41},
     {12, 1, 2, 0, 7, 6, 28, 5, 3, 4, 8, 14, 60, 21, 18, 40, 17, 86, 10, 9, 16, 29, 19, 93, 126, 79, 38, 84, 72, 27, 111, 119},
     {11, 8, 49, 130, 10, 125, 9, 124, 100, 114, 131, 30, 58, 104, 32, 39, 24, 113, 36, 105, 0, 41, 22, 120, 5, 53, 111, 38, 142, 44, 83, 35},
     {50, 70, 47, 118, 85, 57, 106, 0, 45, 7, 64, 90, 81, 14, 2, 134, 28, 62, 86, 55, 69, 1, 78, 119, 68, 56, 18, 67, 16, 60, 29, 21},
     {43, 37, 33, 87, 51, 41, 66, 5, 122, 38, 22, 59, 92, 0, 23, 91, 27, 16, 71, 79, 18, 52, 120, 4, 3, 24, 46, 20, 73, 39, 62, 36},
     {79, 48, 4, 16, 27, 88, 43, 33, 18, 38, 65, 37, 46, 3, 19, 51, 52, 22, 66, 87, 74, 5, 41, 91, 23, 59, 0, 71, 122, 72, 20, 92},
     {32, 100, 10, 8, 30, 104, 24, 44, 39, 113, 83, 103, 1, 7, 22, 53, 115, 63, 135, 121, 26, 35, 34, 5, 0, 108, 137, 90, 91, 45, 2, 130},
     {0, 1, 2, 5, 16, 12, 6, 7, 14, 3, 19, 18, 29, 20, 4, 21, 40, 8, 17, 35, 23, 48, 126, 22, 25, 56, 26, 10, 98, 27, 38, 65},
     {143, 67, 56, 146, 1, 7, 133, 55, 64, 141, 134, 69, 6, 47, 14, 29, 84, 21, 111, 147, 57, 16, 95, 72, 118, 132, 50, 0, 2, 18, 119, 42},
     {1, 7, 67, 14, 133, 111, 8, 84, 0, 21, 2, 47, 64, 132, 55, 10, 95, 147, 119, 42, 16, 5, 72, 56, 4, 3, 6, 29, 9, 25, 18, 30},
     {68, 57, 69, 112, 144, 86, 102, 2, 134, 55, 0, 70, 118, 64, 75, 47, 14, 28, 93, 143, 67, 7, 50, 149, 1, 21, 29, 56, 119, 95, 60, 78},
     {58, 97, 114, 30, 124, 45, 11, 139, 8, 90, 0, 142, 7, 10, 41, 113, 84, 62, 49, 111, 85, 1, 9, 5, 137, 120, 32, 14, 2, 117, 47, 38},
     {23, 66, 18, 79, 38, 20, 43, 27, 16, 88, 46, 59, 126, 37, 87, 12, 73, 92, 3, 5, 48, 0, 19, 54, 2, 51, 28, 1, 41, 65, 122, 22},
     {0, 12, 2, 27, 5, 40, 46, 38, 1, 41, 3, 79, 88, 23, 99, 4, 20, 62, 22, 54, 92, 18, 8, 37, 16, 35, 10, 7, 19, 120, 144, 24},
     {1, 14, 25, 26, 0, 7, 44, 34, 129, 42, 24, 5, 135, 22, 19, 148, 6, 96, 83, 2, 29, 16, 63, 35, 101, 64, 140, 136, 116, 110, 3, 10},
     {12, 1, 2, 27, 3, 4, 38, 5, 7, 8, 18, 16, 46, 6, 0, 40, 41, 10, 79, 23, 88, 9, 20, 22, 14, 19, 37, 92, 48, 126, 28, 21},
     {7, 1, 10, 32, 108, 103, 94, 47, 8, 53, 25, 14, 34, 115, 100, 129, 121, 130, 148, 42, 64, 116, 63, 26, 44, 0, 24, 30, 113, 4, 104, 22},
     {47, 134, 7, 14, 55, 69, 64, 95, 1, 29, 85, 118, 56, 116, 45, 57, 102, 143, 50, 90, 42, 30, 16, 94, 0, 8, 67, 75, 133, 2, 18, 48},
     {12, 1, 2, 0, 7, 6, 28, 8, 14, 5, 3, 4, 40, 21, 17, 18, 60, 86, 16, 93, 126, 10, 9, 29, 99, 38, 119, 25, 19, 54, 27, 84},
     {59, 16, 27, 18, 23, 88, 79, 37, 46, 66, 38, 20, 73, 126, 3, 43, 48, 87, 92, 51, 41, 12, 19, 5, 52, 107, 65, 0, 151, 122, 54, 2},
     {1, 21, 147, 7, 119, 14, 76, 132, 55, 0, 86, 145, 2, 6, 69, 67, 16, 143, 111, 138, 17, 28, 29, 60, 18, 93, 8, 19, 40, 56, 84, 5},
     {144, 86, 112, 2, 68, 102, 69, 0, 149, 93, 75, 28, 57, 55, 145, 60, 21, 67, 99, 134, 143, 40, 146, 119, 82, 110, 62, 6, 29, 26, 78, 14},
     {102, 57, 55, 69, 143, 75, 146, 67, 56, 68, 134, 2, 29, 141, 0, 21, 6, 14, 133, 118, 64, 1, 7, 95, 47, 84, 111, 28, 147, 82, 72, 119},
     {0, 70, 57, 119, 50, 145, 2, 86, 28, 118, 69, 78, 149, 47, 60, 68, 67, 55, 93, 81, 134, 21, 14, 62, 64, 7, 5, 1, 132, 85, 41, 16},
     {51, 5, 43, 71, 122, 87, 41, 37, 91, 39, 0, 22, 33, 36, 38, 24, 66, 120, 62, 2, 80, 16, 92, 10, 59, 4, 27, 23, 35, 79, 8, 3},
     {12, 1, 2, 0, 7, 6, 28, 5, 8, 14, 3, 21, 40, 4, 60, 17, 86, 18, 16, 93, 10, 9, 126, 119, 99, 29, 19, 41, 38, 27, 25, 92},
     {27, 18, 46, 126, 23, 16, 88, 79, 20, 151, 59, 73, 48, 38, 0, 54, 12, 2, 37, 1, 19, 5, 28, 60, 66, 41, 3, 109, 86, 65, 40, 6},
     {48, 79, 4, 33, 16, 74, 65, 38, 88, 27, 91, 52, 18, 36, 22, 19, 46, 0, 37, 3, 51, 5, 71, 39, 72, 43, 24, 41, 92, 87, 2, 10},
     {86, 2, 144, 93, 28, 112, 141, 6, 102, 21, 99, 60, 75, 0, 68, 82, 69, 146, 67, 149, 55, 40, 145, 76, 111, 147, 56, 119, 110, 143, 26, 132},
     {6, 138, 2, 99, 86, 17, 40, 93, 28, 21, 145, 141, 0, 60, 119, 147, 128, 76, 67, 54, 1, 12, 5, 27, 144, 14, 38, 98, 146, 41, 29, 19},
     {1, 8, 0, 10, 2, 29, 7, 5, 3, 56, 4, 25, 14, 152, 63, 32, 65, 72, 96, 42, 34, 108, 48, 9, 26, 16, 84, 103, 67, 148, 22, 129},
     {149, 145, 0, 86, 2, 28, 93, 144, 62, 60, 119, 101, 21, 41, 5, 35, 78, 99, 26, 40, 12, 68, 57, 67, 110, 120, 69, 18, 55, 76, 132, 70},
     {12, 28, 16, 1, 48, 19, 6, 60, 2, 14, 18, 21, 0, 27, 46, 65, 86, 29, 5, 7, 72, 93, 40, 3, 17, 84, 56, 88, 126, 4, 38, 8},
     {1, 8, 5, 10, 7, 24, 2, 62, 0, 41, 22, 122, 120, 9, 4, 3, 32, 87, 11, 37, 38, 83, 100, 44, 25, 104, 16, 26, 39, 80, 14, 6},
     {0, 119, 62, 86, 145, 149, 28, 132, 93, 2, 120, 67, 60, 41, 35, 5, 144, 21, 123, 38, 111, 81, 84, 56, 12, 44, 24, 50, 92, 55, 40, 22},
     {2, 93, 99, 28, 40, 144, 60, 0, 86, 150, 76, 21, 149, 98, 6, 25, 1, 61, 82, 26, 12, 5, 54, 141, 7, 18, 145, 16, 27, 138, 110, 38},
     {24, 8, 10, 22, 32, 35, 100, 5, 1, 53, 0, 7, 71, 80, 30, 123, 83, 104, 51, 11, 2, 39, 44, 113, 9, 62, 25, 103, 34, 101, 43, 41},
     {12, 1, 2, 0, 7, 6, 28, 5, 40, 60, 8, 16, 3, 18, 14, 4, 86, 21, 17, 93, 41, 10, 9, 99, 27, 119, 38, 19, 126, 22, 48, 145},
     {45, 47, 50, 7, 85, 90, 97, 1, 64, 139, 116, 118, 30, 58, 14, 106, 70, 111, 0, 57, 94, 42, 137, 142, 29, 120, 8, 56, 18, 134, 84, 41},
     {12, 0, 2, 5, 27, 38, 1, 46, 41, 40, 79, 144, 3, 22, 88, 23, 28, 60, 99, 62, 6, 24, 26, 7, 4, 16, 10, 35, 37, 18, 14, 20},
     {37, 38, 59, 92, 0, 5, 23, 51, 79, 41, 27, 22, 2, 3, 87, 16, 46, 4, 1, 43, 20, 33, 18, 88, 24, 71, 8, 10, 48, 19, 126, 122},
     {12, 28, 16, 60, 1, 18, 6, 21, 19, 14, 48, 0, 2, 86, 93, 5, 46, 29, 17, 27, 65, 7, 3, 72, 38, 126, 119, 40, 84, 37, 56, 4},
     {0, 2, 5, 1, 16, 6, 27, 28, 18, 38, 60, 7, 14, 21, 46, 40, 86, 41, 19, 48, 93, 8, 3, 79, 22, 4, 10, 37, 62, 23, 24, 111},
     {85, 7, 90, 30, 47, 139, 45, 50, 94, 58, 137, 1, 8, 64, 14, 116, 118, 115, 113, 11, 124, 108, 0, 10, 97, 57, 32, 70, 42, 106, 29, 114},
     {33, 36, 22, 71, 51, 5, 91, 39, 0, 52, 43, 24, 131, 74, 16, 37, 38, 122, 41, 3, 87, 48, 4, 104, 35, 80, 10, 2, 105, 62, 27, 18},
     {12, 1, 27, 2, 0, 16, 3, 28, 46, 18, 4, 6, 5, 72, 21, 79, 38, 7, 14, 60, 88, 8, 65, 19, 48, 29, 23, 40, 22, 20, 86, 126},
     {0, 12, 2, 27, 5, 38, 46, 41, 1, 40, 79, 3, 88, 23, 22, 99, 20, 37, 62, 4, 18, 6, 16, 35, 60, 28, 24, 7, 92, 8, 14, 10},
     {7, 47, 1, 30, 137, 8, 116, 94, 90, 64, 14, 115, 108, 118, 57, 10, 148, 113, 42, 85, 32, 11, 63, 50, 103, 45, 124, 134, 55, 9, 69, 34},
     {55, 7, 1, 29, 56, 143, 64, 47, 67, 133, 14, 146, 95, 72, 84, 8, 116, 111, 6, 134, 141, 21, 65, 0, 69, 30, 16, 45, 85, 42, 50, 10},
     {14, 1, 42, 8, 10, 29, 108, 63, 55, 148, 95, 32, 7, 19, 25, 115, 103, 34, 56, 129, 77, 0, 16, 152, 94, 30, 113, 26, 2, 5, 48, 4},
     {111, 120, 142, 97, 58, 0, 41, 45, 62, 132, 114, 84, 139, 30, 5, 8, 38, 2, 7, 85, 119, 90, 117, 1, 124, 11, 56, 47, 28, 27, 35, 72},
     {1, 0, 14, 2, 6, 5, 16, 19, 7, 29, 42, 18, 3, 25, 12, 35, 21, 8, 26, 17, 40, 4, 20, 48, 109, 99, 22, 96, 55, 101, 10, 61},
     {12, 0, 1, 5, 3, 2, 4, 7, 27, 8, 38, 6, 40, 18, 16, 10, 20, 46, 9, 41, 23, 22, 79, 14, 62, 19, 37, 126, 88, 11, 92, 48},
     {10, 8, 104, 39, 24, 32, 22, 83, 44, 100, 30, 130, 53, 91, 113, 5, 11, 1, 35, 33, 7, 49, 0, 2, 103, 71, 36, 124, 9, 80, 131, 34},
     {1, 7, 0, 14, 8, 34, 5, 25, 35, 26, 6, 63, 10, 123, 2, 16, 103, 19, 44, 32, 135, 121, 108, 80, 62, 30, 115, 94, 149, 144, 53, 18},
     {75, 68, 146, 141, 102, 67, 2, 21, 6, 57, 69, 143, 0, 55, 82, 86, 28, 144, 147, 29, 93, 112, 56, 119, 133, 14, 76, 60, 84, 134, 111, 145},
     {10, 32, 115, 7, 8, 53, 1, 108, 30, 113, 94, 137, 100, 63, 90, 34, 130, 103, 121, 47, 44, 25, 104, 39, 24, 26, 85, 14, 49, 36, 22, 131},
     {39, 24, 10, 22, 8, 130, 91, 104, 83, 49, 5, 33, 100, 11, 0, 35, 32, 131, 71, 36, 9, 44, 53, 2, 80, 51, 30, 1, 41, 7, 43, 62},
     {38, 36, 65, 105, 27, 72, 31, 79, 41, 131, 5, 48, 125, 39, 0, 16, 92, 46, 22, 13, 18, 84, 24, 37, 88, 2, 33, 74, 91, 71, 130, 49},
     {0, 106, 62, 50, 45, 119, 85, 81, 132, 28, 2, 86, 41, 47, 38, 60, 35, 117, 5, 29, 7, 30, 145, 90, 55, 70, 14, 111, 18, 67, 93, 56},
     {0, 2, 5, 1, 3, 25, 19, 26, 4, 34, 29, 10, 22, 16, 8, 7, 24, 14, 48, 65, 53, 18, 6, 77, 44, 56, 72, 61, 121, 21, 136, 40},
     {7, 1, 94, 8, 47, 115, 10, 32, 113, 103, 30, 108, 137, 63, 14, 64, 116, 148, 129, 42, 90, 25, 34, 118, 53, 57, 11, 49, 85, 9, 96, 50},
     {14, 0, 1, 26, 19, 5, 42, 2, 25, 24, 29, 22, 6, 44, 61, 16, 7, 96, 136, 3, 140, 34, 35, 55, 135, 18, 48, 77, 83, 4, 8, 10},
     {1, 7, 14, 0, 25, 6, 34, 5, 26, 16, 63, 2, 19, 8, 35, 101, 108, 29, 94, 10, 18, 42, 123, 144, 129, 47, 61, 21, 3, 62, 149, 4},
     {12, 0, 2, 1, 28, 5, 6, 120, 7, 60, 40, 16, 18, 86, 27, 14, 21, 93, 8, 62, 41, 38, 3, 17, 4, 119, 99, 48, 19, 126, 10, 9},
     {86, 144, 93, 2, 28, 149, 0, 60, 99, 112, 110, 145, 40, 21, 102, 26, 75, 62, 69, 1, 12, 101, 119, 25, 76, 67, 7, 68, 55, 5, 6, 14},
     {8, 30, 10, 32, 113, 49, 115, 137, 124, 103, 45, 90, 7, 139, 11, 1, 58, 53, 130, 94, 108, 100, 9, 63, 85, 125, 34, 47, 0, 24, 44, 104},
     {120, 142, 111, 41, 58, 114, 97, 0, 11, 62, 84, 124, 5, 30, 8, 38, 132, 127, 27, 139, 92, 10, 72, 45, 49, 9, 28, 2, 29, 56, 16, 1},
     {8, 113, 30, 137, 7, 32, 10, 90, 94, 115, 1, 103, 108, 63, 47, 85, 49, 53, 11, 45, 34, 50, 14, 25, 9, 124, 100, 130, 139, 121, 42, 26},
     {64, 7, 14, 47, 134, 55, 1, 42, 95, 69, 116, 90, 94, 30, 8, 29, 56, 137, 45, 108, 85, 10, 57, 16, 102, 143, 118, 19, 63, 32, 11, 50},
     {62, 132, 0, 119, 120, 41, 111, 86, 35, 28, 5, 84, 56, 38, 2, 93, 145, 60, 67, 12, 92, 27, 29, 72, 55, 117, 21, 24, 133, 149, 22, 45},
     {57, 68, 69, 118, 134, 64, 50, 47, 55, 14, 7, 2, 102, 144, 0, 112, 70, 86, 85, 1, 95, 29, 116, 143, 42, 75, 16, 56, 28, 45, 21, 48},
     {0, 12, 2, 1, 5, 28, 6, 40, 60, 27, 7, 38, 16, 14, 86, 18, 93, 41, 62, 46, 99, 35, 8, 23, 3, 17, 22, 21, 10, 19, 79, 20},
     {12, 1, 2, 27, 16, 3, 38, 111, 4, 0, 18, 5, 7, 46, 40, 8, 79, 6, 14, 28, 88, 10, 48, 41, 19, 84, 21, 9, 22, 23, 20, 72},
     {53, 103, 32, 7, 1, 100, 22, 63, 71, 44, 10, 115, 108, 24, 92, 104, 26, 30, 122, 94, 8, 39, 83, 34, 137, 135, 90, 91, 121, 5, 87, 47},
     {87, 37, 41, 0, 22, 38, 2, 92, 1, 24, 4, 8, 3, 59, 10, 5, 39, 23, 71, 79, 122, 27, 16, 46, 33, 7, 91, 20, 18, 51, 9, 120},
     {1, 7, 8, 10, 0, 5, 35, 32, 53, 44, 14, 30, 2, 80, 25, 34, 6, 62, 26, 103, 16, 19, 63, 9, 149, 24, 121, 41, 22, 11, 113, 83},
     {11, 58, 8, 30, 124, 49, 10, 113, 9, 114, 139, 45, 97, 32, 7, 137, 90, 1, 0, 130, 115, 125, 100, 24, 5, 94, 53, 41, 14, 13, 35, 38},
     {125, 105, 9, 36, 131, 49, 8, 130, 39, 11, 10, 5, 22, 38, 41, 104, 0, 31, 13, 24, 27, 16, 2, 72, 65, 91, 48, 32, 84, 18, 100, 74},
     {12, 1, 0, 2, 6, 3, 7, 5, 4, 8, 14, 28, 16, 60, 18, 10, 21, 17, 19, 9, 40, 27, 86, 93, 29, 38, 54, 11, 25, 48, 46, 41},
     {84, 41, 38, 72, 92, 29, 111, 5, 65, 120, 79, 0, 27, 56, 48, 14, 132, 16, 119, 22, 86, 88, 46, 28, 62, 12, 1, 2, 93, 18, 24, 127},
     {99, 28, 40, 60, 2, 93, 138, 0, 98, 17, 86, 54, 76, 12, 27, 1, 21, 144, 128, 38, 5, 14, 46, 18, 25, 16, 109, 6, 41, 145, 7, 29},
     {1, 63, 10, 32, 148, 14, 103, 34, 42, 7, 8, 108, 116, 53, 64, 96, 25, 121, 26, 94, 140, 0, 29, 19, 55, 24, 100, 136, 5, 4, 44, 115},
     {131, 100, 130, 49, 10, 8, 36, 104, 39, 0, 48, 41, 11, 38, 4, 24, 27, 22, 16, 44, 79, 5, 33, 2, 53, 9, 125, 74, 91, 120, 32, 83},
     {36, 39, 131, 74, 4, 91, 22, 33, 125, 104, 130, 48, 10, 24, 16, 5, 49, 8, 100, 105, 79, 0, 9, 65, 71, 2, 18, 83, 31, 11, 19, 44},
     {0, 12, 2, 1, 6, 5, 7, 28, 40, 60, 16, 14, 18, 62, 86, 27, 93, 8, 17, 38, 21, 41, 35, 99, 3, 19, 10, 23, 22, 4, 9, 48},
     {1, 7, 67, 14, 21, 147, 111, 55, 132, 119, 0, 8, 2, 76, 64, 16, 47, 84, 6, 18, 86, 95, 145, 10, 42, 29, 133, 5, 56, 134, 17, 72},
     {69, 55, 47, 134, 102, 143, 7, 57, 118, 95, 14, 64, 29, 56, 1, 50, 75, 67, 146, 2, 0, 133, 68, 16, 21, 6, 141, 85, 116, 18, 72, 65},
     {1, 44, 7, 24, 83, 63, 34, 103, 22, 121, 53, 32, 25, 35, 0, 115, 108, 5, 14, 8, 10, 101, 94, 30, 2, 123, 110, 26, 137, 47, 90, 19},
     {14, 1, 25, 42, 34, 0, 26, 96, 19, 29, 140, 5, 53, 10, 2, 121, 3, 24, 44, 22, 55, 77, 129, 7, 63, 16, 8, 4, 6, 61, 100, 48},
     {30, 90, 7, 8, 137, 94, 85, 1, 47, 113, 115, 108, 45, 139, 124, 11, 10, 32, 50, 58, 103, 14, 63, 64, 9, 116, 49, 42, 25, 148, 0, 53},
     {40, 99, 2, 60, 28, 17, 0, 54, 93, 98, 86, 138, 6, 12, 21, 76, 1, 5, 27, 144, 128, 38, 19, 46, 14, 41, 145, 7, 16, 67, 3, 109},
     {45, 58, 30, 139, 90, 7, 85, 137, 97, 8, 124, 47, 1, 11, 106, 114, 50, 94, 0, 113, 10, 115, 14, 32, 9, 64, 108, 41, 49, 29, 62, 116},
     {14, 42, 10, 1, 63, 96, 32, 25, 34, 8, 129, 29, 0, 103, 55, 19, 26, 53, 77, 5, 95, 2, 4, 7, 3, 16, 148, 56, 18, 24, 121, 108},
     {21, 2, 75, 86, 6, 76, 144, 28, 119, 99, 93, 147, 141, 67, 102, 145, 60, 132, 146, 128, 0, 82, 40, 138, 55, 111, 143, 17, 133, 112, 69, 14},
     {111, 120, 41, 62, 84, 132, 0, 5, 38, 119, 56, 92, 72, 142, 27, 28, 29, 35, 58, 80, 2, 86, 65, 79, 12, 14, 1, 24, 145, 16, 21, 48},
     {146, 67, 141, 69, 133, 21, 6, 143, 57, 55, 111, 147, 56, 1, 14, 132, 7, 2, 134, 102, 0, 119, 29, 84, 76, 64, 86, 72, 28, 68, 47, 75},
     {12, 1, 0, 5, 27, 3, 7, 4, 38, 8, 6, 41, 16, 40, 46, 10, 18, 79, 2, 9, 23, 86, 20, 22, 62, 14, 37, 88, 92, 19, 24, 11},
     {0, 12, 2, 1, 27, 5, 38, 28, 60, 6, 40, 7, 16, 46, 18, 14, 41, 99, 93, 62, 3, 79, 86, 23, 149, 8, 22, 35, 88, 17, 19, 10},
     {141, 6, 21, 67, 147, 102, 146, 2, 76, 119, 132, 69, 55, 111, 86, 75, 28, 133, 143, 0, 1, 145, 14, 128, 56, 99, 17, 60, 29, 93, 84, 68},
     {21, 76, 1, 119, 86, 145, 2, 0, 14, 7, 6, 138, 146, 55, 17, 28, 132, 93, 67, 40, 60, 143, 29, 147, 111, 16, 69, 141, 5, 56, 19, 133},
     {1, 8, 108, 14, 7, 116, 64, 42, 10, 63, 94, 32, 115, 103, 113, 96, 30, 34, 55, 47, 95, 148, 29, 140, 129, 25, 134, 53, 69, 26, 19, 11},
     {12, 1, 3, 5, 4, 2, 0, 7, 8, 38, 27, 16, 18, 6, 10, 20, 41, 40, 79, 46, 9, 23, 22, 88, 92, 37, 14, 24, 62, 19, 48, 99},
     {1, 14, 7, 0, 6, 25, 5, 16, 19, 2, 42, 26, 29, 35, 61, 8, 18, 129, 101, 21, 3, 110, 34, 148, 96, 10, 17, 4, 22, 40, 12, 20},
     {0, 2, 5, 1, 3, 19, 22, 26, 16, 24, 29, 7, 14, 6, 4, 25, 18, 44, 8, 48, 12, 61, 20, 21, 10, 35, 65, 56, 23, 40, 17, 107},
     {1, 7, 8, 29, 56, 0, 10, 14, 2, 42, 72, 5, 4, 65, 3, 30, 84, 94, 67, 9, 25, 133, 111, 11, 32, 108, 16, 63, 21, 96, 26, 48}}};
// endregion

template class OrderTable<3>;
template class OrderTable<4>;

}  // namespace quicktex::s3tc